package leak;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.LockStats;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.TransactionStats;
import com.sleepycat.je.config.BooleanConfigParam;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.dbi.EnvironmentImpl;

public privileged aspect CheckLeaks {
	/**
	 * Debugging support. Check for leaked locks and transactions.
	 */
	private void EnvironmentImpl.checkLeaks() throws DatabaseException {

		/* Only enabled if this check leak flag is true. */
		if (!configManager.getBoolean(EnvironmentParams.ENV_CHECK_LEAKS)) {
			return;
		}

		boolean clean = true;
		StatsConfig statsConfig = new StatsConfig();

		/* Fast stats will not return NTotalLocks below. */
		statsConfig.setFast(false);

		LockStats lockStat = lockStat(statsConfig);
		if (lockStat.getNTotalLocks() != 0) {
			clean = false;
			System.out.println("Problem: " + lockStat.getNTotalLocks()
					+ " locks left");
			txnManager.getLockManager().dump();
		}

		TransactionStats txnStat = txnStat(statsConfig);
		if (txnStat.getNActive() != 0) {
			clean = false;
			System.out.println("Problem: " + txnStat.getNActive()
					+ " txns left");
			TransactionStats.Active[] active = txnStat.getActiveTxns();
			if (active != null) {
				for (int i = 0; i < active.length; i += 1) {
					System.out.println(active[i]);
				}
			}
		}

		clean &= hook_assertLatchClean();

		assert clean : "Lock, transaction, or latch left behind at environment close";
	}

	private boolean EnvironmentImpl.hook_assertLatchClean() {
		return true;
	}
	
	after (EnvironmentImpl env, StringBuffer errors ):
		execution(void EnvironmentImpl.hook_afterDoClose(StringBuffer)) && args(errors) && target(env) {
		try {
			env.checkLeaks();
		} catch (DatabaseException DBE) {
			errors.append("\nException performing validity checks: ");
			errors.append(DBE.toString()).append("\n");
		}
	}
	
	
	public static final BooleanConfigParam EnvironmentParams.ENV_CHECK_LEAKS = new BooleanConfigParam(
			"je.env.checkLeaks", true, // default
			false,// mutable
			"# Debugging support: check leaked locks and txns at env close.");
}
